import requests

from py_behrtech.exceptions import check_status_code
from py_behrtech.Calls.functions import buildParameter


class Mqtt:

    def __init__(self):
        self.username = None
        self.password = None
        self.server_address = None
        self.jwt_token = None

    def brokerBrokerIDGet(self, brokerID: str) -> dict:
        """
        Gets information about the requested MQTT broker

        :param brokerID: Unique broker ID
        :return: Detailed information about the requested MQTT broker
        """

        req = requests.get(url=self.server_address + f"/v2/broker/{brokerID}", verify=False,
                           headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return req.json()
        else:
            check_status_code(req=req)

    def brokerGet(self, returnCount: int = '', offset: int = '') -> dict:
        """
        Gets information about the requested MQTT brokers

        :param returnCount: The amount of MQTT Brokers to be requested. -1 is ALL
        :param offset: The MQTT broker count to start the MQTT Broker data request from
        :return: Detailed information about the requested MQTT brokers
        """

        req = requests.get(url=self.server_address + f"/v2/broker" + buildParameter(params=vars()), verify=False,
                           headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return req.json()
        else:
            check_status_code(req=req)

    def mqttMappingGet(self, returnCount: int = '', offset: int = '') -> dict:
        """
        Gets information about the requested MQTT mappings

        :param returnCount: The amount of MQTT mappings to be requested. -1 is ALL
        :param offset: The MQTT mappings count to start the MQTT mappings data request from
        :return: Detailed information about the requested MQTT mappings
        """

        req = requests.get(url=self.server_address + f"/v2/mqttmapping" + buildParameter(params=vars()), verify=False,
                           headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return req.json()
        else:
            check_status_code(req=req)

    def mqttMappingMappingIDGet(self, mappingID: str) -> dict:
        """
        Gets information about the requested MQTT mapping

        :param mappingID: Unique mapping ID
        :return: Detailed information about the requested MQTT mapping
        """

        req = requests.get(url=self.server_address + f"/v2/mqttmapping/{mappingID}",
                           verify=False, headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return req.json()
        else:
            check_status_code(req=req)




from requests import Response


def check_status_code(req: Response, messages: dict = None):
    """
    Check status code of requests response to raise the correct exception

    :param req: The requests Response of your requests call
    :param messages: Custom messages for error codes
    """

    if messages is None:
        messages = {}
    if req.status_code == 400:
        raise QueryError(
            status_code=400, url=req.url,
            message="Endpoint is invalid or was built incorrectly" if 400 not in messages else messages[400]
        )
    elif req.status_code == 401:
        raise JWTError(
            status_code=401,
            message="Missing JWT Access token or invalid credentials provided" if 401 not in messages else messages[401]
        )
    elif req.status_code == 403:
        raise PermissionsError(
            status_code=403,
            message="User doesn't have the correct permissions" if 403 not in messages else messages[403]
        )
    elif req.status_code == 404:
        raise QueryError(
            status_code=404, url=req.url,
            message="Endpoint is invalid or was built incorrectly" if 404 not in messages else messages[404]
        )
    elif req.status_code == 424:
        raise DependencyError(
            status_code=424,
            message="The request could not be performed because the requested action depends on another action and "
                    "that action failed." if 424 not in messages else messages[424]
        )
    elif req.status_code == 500:
        raise InternalServerError(
            status_code=500,
            message="Internal server error from the API" if 500 not in messages else messages[500]
        )


class PyBehrtechException(Exception):
    """Base exception used by this module."""
    pass


class JWTError(PyBehrtechException):
    """A request for the JWT failed"""

    def __init__(self, status_code, message):
        self.status_code = status_code
        PyBehrtechException.__init__(self, status_code, message)


class PermissionsError(PyBehrtechException):
    """User of request doesn't have permissions"""

    def __init__(self, status_code, message):
        self.status_code = status_code
        PyBehrtechException.__init__(self, status_code, message)


class QueryError(PyBehrtechException):
    """Request to endpoint is invalid or was built incorrectly"""

    def __init__(self, status_code, url, message):
        self.status_code = status_code
        self.url = url
        PyBehrtechException.__init__(self, status_code, url, message)


class DependencyError(PyBehrtechException):
    """Failed dependency, the request could not be preformed"""

    def __init__(self, status_code, message):
        self.status_code = status_code
        PyBehrtechException.__init__(self, status_code, message)


class InternalServerError(PyBehrtechException):
    """Internal Server Error from the API"""

    def __init__(self, status_code, message):
        self.status_code = status_code
        PyBehrtechException.__init__(self, status_code, message)



import requests

from py_behrtech.exceptions import check_status_code
from py_behrtech.Calls.functions import buildParameter


class Azure:

    def __init__(self):
        self.username = None
        self.password = None
        self.server_address = None
        self.jwt_token = None

    def azureFunctionDelete(self, deleteAll: bool = False) -> dict:
        """
        Deletes all Azure functions from the Service Center. Iterates over all existing functions, if a function still
        has mappings referred to it, the task will stop, resulting in some functions being deleted and some not.
        No guarantee can be given if all functions without mappings are actually deleted.

        :param deleteAll: Verifies deletion of all Azure functions is on purpose
        :return: Number of Azure functions deleted
        """

        req = requests.delete(url=self.server_address + f"/v2/azurefunction?deleteAll={deleteAll}", verify=False,
                              headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return req.json()
        else:
            check_status_code(req=req)

    def azureFunctionIdDelete(self, functionId: str) -> bool:
        """
        Delete a single Azure function from the Service Center.

        :param functionId: Unique Azure function ID
        :return: Bool if Azure function has been successfully deleted or not
        """

        req = requests.delete(url=self.server_address + f"/v2/azurefunction/{functionId}", verify=False,
                              headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return True
        else:
            check_status_code(req=req)

    def azureFunctionIdGet(self, functionId: str) -> dict:
        """
        Gets information about the requested Azure function

        :param functionId: Unique Azure function ID
        :return: Detailed information about the requested Azure function
        """

        req = requests.get(url=self.server_address + f"/v2/azurefunction/{functionId}", verify=False,
                           headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return req.json()
        else:
            check_status_code(req=req)

    def azureFunctionGet(self, returnCount: int = '', offset: int = '') -> dict:
        """
        Gets information about all Azure functions

        :param returnCount: The amount of Azure functions to be requested. -1 is ALL
        :param offset: The Azure functions count to start the Azure function data request from
        :return: Detailed information about the requested Azure functions
        """

        req = requests.get(url=self.server_address + f"/v2/azurefunction" + buildParameter(params=vars()), verify=False,
                           headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return req.json()
        else:
            check_status_code(req=req)

    def azureMappingDelete(self, deleteAll: bool = False) -> dict:
        """
        Deletes all Azure mappings from the Service Center.

        :param deleteAll: Verifies deletion of all Azure mappings is on purpose
        :return: Number of Azure mappings deleted
        """

        req = requests.delete(url=self.server_address + f"/v2/azuremapping?deleteAll={deleteAll}", verify=False,
                              headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return req.json()
        else:
            check_status_code(req=req)

    def azureMappingGet(self, returnCount: int = '', offset: int = '', azureFunctionID: str = '') -> dict:
        """
        Gets information about the requested Azure mappings

        :param returnCount: The amount of Azure mappings to be requested. -1 is ALL
        :param offset: The Azure mapping count to start the Azure mapping data request from
        :param azureFunctionID: Unique Azure function ID
        :return: Detailed information about the requested Azure mappings
        """

        req = requests.get(url=self.server_address + f"/v2/azuremapping" + buildParameter(params=vars()), verify=False,
                           headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return req.json()
        else:
            check_status_code(req=req)

    def azureMappingIdDelete(self, mappingId: str) -> bool:
        """
        Delete a single Azure mapping from the Service Center.

        :param mappingId: Unique Azure mapping ID
        :return: Bool if Azure mapping has been successfully deleted or not
        """

        req = requests.delete(url=self.server_address + f"/v2/azuremapping/{mappingId}", verify=False,
                              headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return True
        else:
            check_status_code(req=req)

    def azureMappingIdGet(self, mappingId: str) -> dict:
        """
        Gets information about the requested Azure mapping

        :param mappingId: Unique Azure mapping ID
        :return: Detailed information about the requested Azure mapping
        """

        req = requests.get(url=self.server_address + f"/v2/azuremapping/{mappingId}", verify=False,
                           headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return req.json()
        else:
            check_status_code(req=req)



import requests

from py_behrtech.exceptions import check_status_code
from py_behrtech.Calls.functions import buildParameter
from py_behrtech.Parsers.nodeParser import NodeParser


class Nodes:

    def __init__(self):
        self.username = None
        self.password = None
        self.server_address = None
        self.jwt_token = None

    def nodesEpEuiGet(self, epEui: str) -> NodeParser:
        """
        Gets information on a configured node with the matching epEui

        :param epEui: The unique epEui of the node to be requested
        :return: Information on the requested node
        """

        req = requests.get(url=self.server_address + f"/v2/nodes/{epEui}", verify=False,
                           headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return NodeParser(req=req)
        else:
            check_status_code(req=req)

    def nodesEpEuiTxDataGet(self, epEui: str) -> dict:
        """
        Gets downlink information on a configured node with the matching epEui

        :param epEui: Unique epEui of the node to be requested
        :return: Information on the requested node
        """

        req = requests.get(url=self.server_address + f"/v2/nodes/{epEui}/txdata", verify=False,
                           headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return req.json()
        else:
            check_status_code(req=req)

    def nodesEpEuiTxDataIdGet(self, epEui: str, messageId: str) -> dict:
        """
        Gets a specific requested downlink message

        :param epEui: Unique epEui of the node to request downlink message for
        :param messageId: Unique ID of the message to be requested
        :return: Message information of the requested node
        """

        req = requests.get(url=self.server_address + f"/v2/nodes/{epEui}/txdata/{messageId}", verify=False,
                           headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return req.json()
        else:
            check_status_code(req=req)

    def nodesGet(self, returnCount: int = '', offset: int = '', sensor_type: str = '') -> NodeParser:
        """
        Gets information on all configured nodes

        :param returnCount: The amount of nodes to be requested. (-1 for all)
        :param offset: Node amount to start the request from
        :param sensor_type: Node type to be requested
        :return: Parser object of configured nodes
        """

        req = requests.get(url=self.server_address + f"/v2/nodes" + buildParameter(params=vars()), verify=False,
                           headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return NodeParser(req=req)
        else:
            check_status_code(req=req)




import json

from typing import Union
from requests import Response
from py_behrtech.Parsers.defaults import Defaults


class NodeParser(Defaults):
    # TODO: setup for downlink messages from nodesEpEuiTxDataGet and nodesEpEuiTxDataIdGet when bi-directional is setup

    def __init__(self, req: Response):

        def getCount():
            count = json.loads(self.data).get('count')
            return count if count is not None else 1

        Defaults.__init__(self)
        self.response: Response = req
        self.data: str = req.text
        self.count = getCount()
        self.nodes = json.loads(self.data).get('nodes')

    def getEpEui(self) -> Union[list, dict]:
        """
        Gets the epEui of nodes

        :return: The epEui of nodes
        """

        if self.nodes:
            return [{'name': x.get('name'), 'epEui': x.get('epEui')} for x in self.nodes]
        else:
            data = json.loads(self.data)
            return {'name': data.get('name'), 'epEui': data.get('epEui')}

    def getType(self) -> Union[list, dict]:
        """
        Gets the type of nodes

        :return: The type of nodes
        """

        if self.nodes:
            return [{'name': x.get('name'), 'type': x.get('type')} for x in self.nodes]
        else:
            data = json.loads(self.data)
            return {'name': data.get('name'), 'type': data.get('type')}

    def getMapping(self) -> Union[list, dict]:
        """
        Gets the mapping of nodes

        :return: The mapping of nodes
        """

        if self.nodes:
            return [{'name': x.get('name'), 'mapping': x.get('mapping')} for x in self.nodes]
        else:
            data = json.loads(self.data)
            return {'name': data.get('name'), 'mapping': data.get('mapping')}

    def getAzureMapping(self) -> Union[list, dict]:
        """
        Gets the Azure mapping of nodes

        :return: The Azure mapping of nodes
        """

        if self.nodes:
            return [{'name': x.get('name'), 'Azure Mapping': x.get('mapping').get('azureMapping')} for x in self.nodes]
        else:
            data = json.loads(self.data)
            return {'name': data.get('name'), 'Azure Mapping': data.get('mapping').get('azureMapping')}

    def getMqttMapping(self) -> Union[list, dict]:
        """
        Gets the MQTT mapping of nodes

        :return: The MQTT mapping of nodes
        """

        if self.nodes:
            return [{'name': x.get('name'), 'MQTT Mapping': x.get('mapping').get('mqttMapping')} for x in self.nodes]
        else:
            data = json.loads(self.data)
            return {'name': data.get('name'), 'MQTT Mapping': data.get('mapping').get('mqttMapping')}

    def getPluginMapping(self) -> Union[list, dict]:
        """
        Gets the plugin mapping of nodes

        :return: The plugin mapping of nodes
        """

        if self.nodes:
            return [{
                'name': x.get('name'),
                'Plugin Mapping': x.get('mapping').get('pluginMapping')
            } for x in self.nodes]
        else:
            data = json.loads(self.data)
            return {'name': data.get('name'), 'Plugin Mapping': data.get('mapping').get('pluginMapping')}
